﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Speech.Recognition;

namespace SpeechEngineLibrary
{
    /// <summary>
    /// The SpeechElement corresponds to one level of speech recognition, 
    /// that is one SpeechElement should match one logical meaning. 
    /// Pronounciations are both clarifications as well as alternate wordings.
    /// </summary>
    public class SpeechElement
    {
        public SpeechElement Parent { get; private set;  }
        public String Name { get; set; }
        public Object TagObject { get; set; }
        List<String> _Pronounciations = new List<string>();
        public ICollection<String> Pronounciations { get { return _Pronounciations; } }

        public SpeechEngine Engine { get; private set; }
        
        List<SpeechElement> _FollowUps = new List<SpeechElement>();
        public IEnumerable<SpeechElement> FollowUps { get { return _FollowUps; } }

        public void AddFollowUp(SpeechElement followUp)
        {
            Debug.Assert(!Engine.SpeechEnabled, "Can't change rules while speech is enabled");
            _FollowUps.Add(followUp);
        }

        public void AddPronounciation(String pronounciation)
        {
            Debug.Assert(!Engine.SpeechEnabled, "Can't change rules while speech is enabled");
            _Pronounciations.Add(pronounciation);
        }

        public SpeechElement(SpeechElement parent, String text)
        {
            Debug.Assert(parent != null);
            Parent = parent;
            Engine = parent.Engine;
            Name = text;
            AddPronounciation(text);
            parent.AddFollowUp(this);
        }

        public SpeechElement(SpeechEngine engine, String text)
        {
            Debug.Assert(engine != null);
            Engine = engine;
            Name = text;
            AddPronounciation(text);
            Engine.AddCommand(this);
        }

        public SpeechElement this[string word]
        {
            get
            {
                if (String.IsNullOrEmpty(word)) return this;

                foreach (SpeechElement followUp in _FollowUps)
                {
                    if (followUp.Match(ref word))
                    {
                        return followUp[word];
                    }
                }
                return null;
            }
        }

        public delegate void SpeechMatch(SpeechElement match);
        public event SpeechMatch OnMatch;
        public void FireMatch()
        {
            if (OnMatch != null)
            {
                OnMatch(this);
            }
        }

        internal void RebuildGrammar(GrammarBuilder CommandGrammarBuilder)
        {
            CommandGrammarBuilder.Append(new Choices(_Pronounciations.ToArray()));
            if (_FollowUps.Count != 0)
            {
                Choices followups = new Choices();
                foreach (SpeechElement followUp in _FollowUps)
                {
                    GrammarBuilder FollowUpGrammarBuilder = new GrammarBuilder();
                    followUp.RebuildGrammar(FollowUpGrammarBuilder);
                    followups.Add(FollowUpGrammarBuilder);
                }
                CommandGrammarBuilder.Append(followups);
            }
        }

        internal bool Match(ref string ID)
        {
            foreach (String pronounciation in Pronounciations)
            {
                if( ID==pronounciation)
                {
                    ID = null;
                    return true;
                }
                if( ID.StartsWith(pronounciation+" "))
                {
                    ID = ID.Substring(pronounciation.Length + 1);
                    return true;
                }
            }
            return false;
        }
    }
}
