﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Speech.Recognition;

namespace SpeechEngineLibrary
{
    /// <summary>
    /// Main container for Speech Recognition Logic, it calls events on matched commands and take no action on its own.
    /// Warning comments are from an msdn sample and may be íncorrect/imprecise.
    /// </summary>
    public class SpeechEngine
    {
        public SpeechRecognitionEngine appRecognizer { get; private set; }

        private bool speechEnabled = false;
        private bool speechInitialized = false;
        /// <summary>
        ///     Property SpeechEnabled is read/write-able. When it's set to
        ///     true, speech recognition will be started. When it's set to
        ///     false, speech recognition will be stopped.
        /// </summary>
        public bool SpeechEnabled
        {
            get
            {
                return speechEnabled;
            }
            set
            {
                if (speechEnabled != value)
                {
                    speechEnabled = value;

                    if (speechEnabled)
                    {
                        EnableSpeech();
                    }
                    else
                    {
                        DisableSpeech();
                    }
                }
            }
        }

        public SpeechElement this[string ID]
        {
            get
            {
                foreach (SpeechElement se in _Commands)
                {
                    if( se.Match(ref ID) )
                    {
                        return se[ID];
                    }
                }
                return null;
            }
        }
        
        List<SpeechElement> _Commands = new List<SpeechElement>();
        internal void AddCommand(SpeechElement speechElement)
        {
            Debug.Assert(!SpeechEnabled, "Can't change rules while speech is enabled");
            _Commands.Add(speechElement);
        }

        /// <summary>
        ///     This function will create the main SpSharedRecoContext object 
        ///     and other required objects like Grammar and rules. 
        ///     In this sample, we are building grammar dynamically since 
        ///     listbox content can change from time to time.
        ///     If your grammar is static, you can write your grammar file 
        ///     and ask SAPI to load it during run time. This can reduce the 
        ///     complexity of your code.
        /// </summary>
        private void InitializeSpeech()
        {
            Debug.WriteLine("Initializing SAPI objects...");

            try
            {
                appRecognizer = new SpeechRecognitionEngine();
                appRecognizer.SetInputToDefaultAudioDevice();
                appRecognizer.SpeechHypothesized +=new EventHandler<SpeechHypothesizedEventArgs>(appRecognizer_SpeechHypothesized);
                appRecognizer.SpeechRecognized +=new EventHandler<SpeechRecognizedEventArgs>(appRecognizer_SpeechRecognized);
            
                // Now add existing list items to the ruleListItems
                RebuildGrammar();

                appRecognizer.RecognizeAsync(RecognizeMode.Multiple);

                // Now we can activate the top level rule. In this sample, only 
                // the top level rule needs to activated. The ListItemsRule is 
                // referenced by the top level rule.
                speechInitialized = true;
            }
            catch (Exception e)
            {
                throw new Exception("Exception caught when initializing SAPI."
                    + " This application may not run correctly.\r\n\r\n"
                    + e.ToString(), e);
            }
        }

        void appRecognizer_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
        }

        void appRecognizer_SpeechHypothesized(object sender, SpeechHypothesizedEventArgs e)
        {
        }

        private void RebuildGrammar()
        {
            appRecognizer.UnloadAllGrammars();
            foreach (SpeechElement command in _Commands)
            {
                GrammarBuilder CommandGrammarBuilder = new GrammarBuilder();
                command.RebuildGrammar(CommandGrammarBuilder);
                String s = CommandGrammarBuilder.DebugShowPhrases;
                Grammar CommandGrammar = new Grammar(CommandGrammarBuilder);
                CommandGrammar.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(PlayCommand_SpeechRecognized);
                appRecognizer.LoadGrammar(CommandGrammar);
            }
        }

        void PlayCommand_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            SpeechElement se = this[e.Result.Text];
            if (se != null) se.FireMatch();
        }

        /// <summary>
        ///     EnableSpeech will initialize all speech objects on first time,
        ///     then rebuild grammar and start speech recognition.
        /// </summary>
        /// <returns>
        ///     true if speech is enabled and grammar updated.
        ///     false otherwise, which happens if we are in design mode.
        /// </returns>
        /// <remarks>
        ///     This is a private function.
        /// </remarks>
        private bool EnableSpeech()
        {
            Debug.Assert(speechEnabled, "speechEnabled must be true in EnableSpeech");

            if (speechInitialized == false)
            {
                InitializeSpeech();
            }
            else
            {
                RebuildGrammar();
            }
            return true;
        }

        /// <summary>
        ///     This is a private function that stops speech recognition.
        /// </summary>
        /// <returns></returns>
        private bool DisableSpeech()
        {
            Debug.Assert(speechInitialized,
                         "speech must be initialized in DisableSpeech");

            if (speechInitialized)
            {
                appRecognizer.RecognizeAsyncCancel();
            }

            return true;
        }

    }
}
